import The from "./The.js";
import dom from "./elements/Dom.js";
import Tabable from "./elements/Tabable.js";
import InfoPage from "./pages/InfoPage.js";
import RequestPage from "./pages/RequestPage.js";
import GlobalsPage from "./pages/GlobalsPage.js";
import FunctionsPage from "./pages/FunctionsPage.js";
import ResponsePage from "./pages/ResponsePage.js";
import ErrorsPage from "./pages/ErrorsPage.js";
import PluginsPage from "./pages/PluginsPage.js";

export default class Request {

	constructor(request_id, li) {
		// console.log('request::construct', request_id);
		this.id = request_id;
		this.li = li;
		this.name = 'req-' + request_id;

		this.Info = new InfoPage(this, this.name);
		this.Request = new RequestPage(this, this.name);
		this.Response = new ResponsePage(this, this.name);
		this.Functions = new FunctionsPage(this, this.name);
		this.Globals = new GlobalsPage(this, this.name);
		this.Plugins = {};

		this.tabs = new Tabable(
			{
				Info: this.Info.get(),
				Request: this.Request.get(),
				Response: this.Response.get(),
				Functions: this.Functions.get(),
				Globals: this.Globals.get(),
				Errors: false
			},
			false,
			this.name
		);

		this.content = this.tabs.get();
	}

	get() {
		// console.log('request::get', this.id);
		return this.content;
	}

	show() {
		// console.log('request::show', this.id);

		if (this.content.style.zIndex != 10) {
			this.content.style.zIndex = 10;
			this.tabs.initClick("Info");
		}
	}

	hide() {
		// console.log('request::hide', this.id);
		this.content.style = "z-index: 0";
	}

	delete() {
		this.tabs = null;
	}

	// process incoming event data
	add(data) {
		let type = data.type || '';
		// console.log('request::add', type);
		// TODO: rethink adding data to element and enabling tab logic
		switch(type) {
			case 'RINIT':
				this.Info.init(data);
				//TODO: this should be in RequestList probably
				dom.props(
					this.li,
					{
						text: data.method + ' ' + data.target.replace(The.protocol() + "://" + The.host(), '').replace(':80', ''),
						class: "listen",
						data: { action: "showRequest" }
					}
				);
				this.tabs.enable("Info");
				break;
			case 'REQUEST_HEADERS':
				this.Request.headers(data.headers);
				this.tabs.enable("Request");
				break;
			case 'RAW_POST_DATA':
				this.Request.rawPost(data.data);
				this.tabs.enable("Request");
				break;
			case 'ERROR':
				if (!this.Errors) {
					this.Errors = new ErrorsPage(this, this.name);
					this.tabs.append("Errors", this.Errors.get());
				}
				this.Errors.add(data);
				this.tabs.enable("Errors");
				break;
			case 'STORAGE':
				let namespace = data.name_space || "unknown";
				namespace = namespace[0].toUpperCase() + namespace.slice(1); // uppercase first char

				if (namespace in this.Plugins === false) {
					this.Plugins[namespace] = new PluginsPage(this, namespace, this.name);
					this.tabs.append(namespace, this.Plugins[namespace].get());
				}
				this.Plugins[namespace].add(data);
				this.tabs.enable(namespace);
				break;
			case 'SUPERGLOBAL':
				this.Globals.add(data.name, data.values);
				this.tabs.enable("Globals");
				break;
			case 'RESPONSE_HEADERS':
				this.Response.headers(data.headers);
				this.tabs.enable("Response");
				break;
			case 'RESPONSE_BODY':
				this.Response.body(data.body);
				this.tabs.enable("Response");
				break;
			case 'FUNCTIONS':
				this.Functions.add(data.functions);
				let func = data.functions[0];
				if (func.function_name && func.total_duration_inclusive && this.Info) {
					this.Info.functionsTime(func.total_duration_inclusive);
				}
				this.tabs.enable("Functions");
				break;
			case 'RSHUTDOWN':
				this.Info.shutdown(data);
				for (let plugin in this.Plugins) {
					this.Plugins[plugin].build();
				}
				break;
			default:
				console.log('Unknown ' + data.type, data);
		}
	}
}