import dom from "./Dom.js";

export default class Collapsible {

	constructor(params, prefix) {
		this.startOpen = !!(params.open || false);
		this.data = {};
		this.timer = false;
		this.content = dom.div({
			id: prefix + '-tree',
			class: "collapsible"
		});
	}

	static toggle(btn) {
		let content = btn.nextElementSibling;
		content.classList.toggle("hidden");
		btn.classList.toggle("active");
	}

	get() {
		return this.content;
	}

	add(key, data) {
		if (this.timer) {
			clearTimeout(this.timer);
			this.timer = false;
		}
		if (typeof(key) == "object") {
			for (let k in key) {
				this.addToData(k, key[k], this.data);
			}
		}
		else {
			this.addToData(key, data, this.data);
		}
		// console.log('Collapsible::add ---', this.data);

		// avoiding rebuild for every event
		this.timer = setTimeout(
			() => {
				this.content.innerHTML = '';
				let nodes = [];
				for (let key in this.data) {
					nodes = this._buildNode(key, this.data[key]);

					if (this.startOpen) {
						for (let c in nodes) {
							nodes[c].classList.toggle(nodes[c].tagName === "DL" ? "active" : "hidden");
						}
					}
					this.content.append(...nodes);
				}
			},
			100
		);
	}

	_buildNode(name, data) {
		let nodes = [];
		let value = ((data === null || data === undefined) ? null : (data.value || data));
		// let oldValue = data.old_value || false;
		if (value && typeof (value) === "object") {
			nodes.push(
				dom.dl({
					class: "listen",
					data: { action: "collapsible" },
					append: [
						dom.dt({ text: name }),
						dom.dd({
							text: Array.isArray(value)
								? "Array [" + value.length + "]"
								: "Object [" + Object.keys(value).length + "]"
						})
					],
				})
			);
			nodes.push(
				this._buildObject(value)
			);
		}
		else {
			nodes.push(
				isNaN(name)
				?
				dom.dl({
					append: [
						dom.dt({ text: name }),
						dom.dd({ append: dom.code({ text: JSON.stringify(value) })})
					]
				})
				: // omitting numeric index for array and show just value
				dom.dl({
					append: [
						dom.dt({ text: JSON.stringify(value) }),
						dom.dd({ text: '' })
					]
				})
			);
		}

		return nodes;
	}

	_buildObject(data) {
		let div = dom.div({ class: "hidden" });

		for (var key in data) {
			let nodes = this._buildNode(key, data[key]);
			div.append(...nodes);
		}

		return div;
	}

	addToData(name, data, struct) {
		let type = Array.isArray(data) ? 'array' : typeof (data);

		switch (type) {
			case "object":
				if (name in struct === false) struct[name] = {};
				for (let key in data) {
					this.addToData(key, data[key], struct[name]);
				}
				break;
			case "array":
				if (name in struct === false) struct[name] = [];
				for (let key in data) {
					struct[name] = this.addToData(struct[name].length, data[key], struct[name]);
				}
				break;
			default:
				struct[name] = data;
		}

		return struct;
	}
}
