import dom from "./Dom.js";

export default class Tabable {

	/**
	 *
	 * @param {object} initTabs
	 */
	constructor(initTabs, title, prefix) {
		// console.log('tabable::construct', initTabs);
		this.initialized = false;
		this.tabs = {};
		this.name = prefix + '-' + (title || 'main').toLowerCase().replace(' ', '');
		// initTabs; // {Info: element,Request: element,Response: element,Functions: element,Globals: element,Errors: false}
		this.slider = dom.div({ class: "tab-slider" });

		this.toolbar = dom.div({
			id: this.name + '-tb', // for Selenium
			class: "toolbar"
		});
		if (title) dom.span({ text: title, class: 'title', appendTo: this.toolbar });
		this.boxes = dom.div({/*  class: "tab-box", */ append: this.slider });

		for (let name in initTabs) {
			this.tabs[name] = {};
			this.tabs[name].btn = this._button(name);
			this.tabs[name].btn.classList.add((initTabs[name] ? "disabled" : "hidden"));
			this.tabs[name].box = this._box(name, initTabs[name]);
			this.tabs[name].box.style = "z-index: " + (this.boxes.childElementCount == 1 ? '10' : '1');
		}

		this.content = dom.div({
			id: this.name, // for Selenium
			append: [this.toolbar, this.boxes]
		});

		// console.log('tabable::init --- ', this.content);
	}

	get() {
		// console.log('tabable::get', this.content);
		return this.content;
	}

	_button(name) {
		return dom.create(
			'button',
			{
				id: this.name + '-btn-' + name.toLowerCase().replace(' ', ''), // for Selenium
				class: "tab-btn",
				text: name,
				data: { action: "showTab" },
				appendTo: this.toolbar
			}
		);
	}
	_box(name, pageElement) {
		return dom.create(
			'div',
			{
				id: this.name + '-pg-' + name.toLowerCase().replace(' ', ''), // for Selenium
				class: "tab",
				data: { tab: name },
				style: "z-index: 1",
				appendTo: this.boxes,
				append: pageElement
			}
		);
	}

	enable(name) {
		// console.log('tabable::enable', name);
		if (name in this.tabs) {
			// console.log('tabable::enable', name);
			this.tabs[name].btn.classList.remove("disabled", "hidden");
			this.tabs[name].btn.classList.add("listen");
		}
		else {
			throw("Tabable::enable unable", name);
		}
	}

	append(name, pageElement) {
		// console.log('tabable::append', name);
		if (name in this.tabs) {
			this.tabs[name].box.innerHTML = '';
			this.tabs[name].box.appendChild(pageElement);
		}
		else {
			this.tabs[name] = {
				btn: this._button(name),
				box: this._box(name, pageElement)
			};
			// throw("tabable: unable to append to non-tab " + name);
		}
	}

	initClick(name) {
		if (!this.initialized) {
			this.initialized = true;
			Tabable.click(this.tabs[name].btn);
		}
	}

	static click(btn) {
		// console.log('tabable::click', btn);
		let name = btn.innerText;
		let scope = btn.parentElement.nextElementSibling;
		let tabs = scope.children;
		let tabKey = false;
		for (let key = 1; key < tabs.length; key ++) {
			// console.log('tab key', key, tabs[key].dataset);
			if (name == tabs[key].dataset.tab) {
				tabKey = key;
			}
			else {
				tabs[key].style = "z-index: 1";
				// this.tabs[key].box.style.visibility = "hidden";
			}
		}
		if (tabKey) {
			tabs[tabKey].style = "z-index: 10";

			// add title for tooo long texts
			let tds = tabs[tabKey].getElementsByTagName('td');
			for (let i = 0; i < tds.length; i++) {
				if (!tds[i].hasAttribute('title') && tds[i].offsetWidth < tds[i].scrollWidth) {
					tds[i].setAttribute('title', tds[i].innerText);
				}
			}
		}
		// this.tabs[name].box.style.visibility = "visible";

		// animate tab activation visualization slider
		let slider = scope.children[0];
		slider.style.width = btn.offsetWidth + "px";
		slider.style.transform = "translateX(" + btn.offsetLeft + "px)";
	}

	clear() {
		// console.log('tabable::clear');
		this.content.innerHTML = "";
		this.tabs = {};
	}
}