import Sortable from "../elements/Sortable.js";

export default class ErrorsPage {
	constructor(request, prefix) {
		// console.log('ErrorsPage::construct', request);
		this.Request = request;

		this.table = new Sortable(
			{
				Type: {
					width: "30%",
					tooltip: "Entry type"
				},
				Message: {
					width: "40%",
					tooltip: "Entry message"
				},
				Filename: {
					width: "30%",
					tooltip: "Entry filename"
				}
			},
			prefix + '-pg-errors'
		);
	}

	get() {
		return this.table.get();
	}

	add(err) {
		// console.log('ErrorsTab::add');

		let type = err.class_name || err.error_type_str || err.type || 'unknown';
		let message = err.message || '-';
		if (type === "ERROR") {
			// remove possible Stack trace from error message
			let pos = message.lastIndexOf("\nStack trace:");
			if (pos > 0) {
				message = message.slice(0, pos);
			}
		}
		// replace carriage returns with symbol and emphasice longer spaces
		message = message.replaceAll(/\r\n|\r|\n/g, "&crarr;").replaceAll("  ","&nbsp;&nbsp;");
		let file_name = err.file_name || err.filename || '';
		let line_no = err.line_no || err.lineno || '';
		let row = {
			'Type': {
				value: type,
				tooltip: type
			},
			'Message': message,
			'Filename': {
				value: file_name ? (file_name.split('/').pop() + ":" + line_no) : '',
				tooltip: file_name ? (file_name + ":" + line_no) : ''
			}
		};

		this.table.addRows([row]);
	}
}