import Sortable from "../elements/Sortable.js";

export default class FunctionsPage {
	constructor(request, prefix) {
		// console.log('FunctionsPage::construct', request);
		this.Request = request;

		this.table = new Sortable(
			{
				Function: {
					width: "35%",
					tooltip: "Function name"
				},
				Count: {
					width: "10%",
					tooltip: "Number of function calls"
				},
				Inclusive: {
					width: "10%",
					title: "Duration Inclusive (ms)",
					tooltip: "Time spent including calls to other functions or children"
				},
				Exclusive: {
					width: "10%",
					title: "Duration Exclusive (ms)",
					tooltip: "Time spent not including other functions or children"
				},
				Average: {
					width: "10%",
					tooltip: "Duration (Inclusive) / call count"
				},
				What: {
					width: "25%",
					title: "Defined At",
					tooltip: "File and line number for function"
				}
			},
			prefix + '-pg-functions'
		);
	}

	get() {
		return this.table.get();
	}

	add(data) {
		// console.log('FunctionsTab::add');

		let rows = [];
		for (var key in data) {
			let func = data[key];
			if (parseInt(func.times_called) > 0) {
				let row = {
					'Function': func.function_name || '',
					'Count': func.times_called || '',
					'Inclusive': (func.total_duration_inclusive || 0) ? parseFloat(func.total_duration_inclusive).toFixed(3) : '0.00',
					'Exclusive': (func.total_duration_exclusive || 0) ? parseFloat(func.total_duration_exclusive).toFixed(3) : '0.00',
					'Average': parseInt(func.times_called) > 0 ? (parseFloat(func.total_duration_inclusive || 0) / parseInt(func.times_called)).toFixed(3) : '0.00',
					'What': {
						value: (func.file_name || false) ? (func.file_name.split('/').pop() + ":" + func.line_start) : '',
						tooltip: (func.file_name || false) ? (func.file_name + ":" + func.line_start) : ''
					}
				};
				rows.push(row);
			}
		}

		this.table.addRows(rows);
	}
}