import dom from "../elements/Dom.js";
import The from "../The.js";

let template = {
	page: (info, time, mem, process) => `
	<table class="graph-table">
		<colgroup>
			<col class="labels" />
			<col class="bars" />
			<col class="value" />
		</colgroup>
		<tr class="graph-table-header">
			<td colspan="2">Request</td>
			<td>Status</td>
		</tr>
		<tr class="graph-footer">
			<td colspan="2">${info.method} ${info.url}</td>
			<td>${info.code}</td>
		</tr>
		<tr class="graph-table-header">
			<td colspan="3">
				<hr class="break" />
			</td>
		</tr>
		<tr class="graph-table-header">
			<td>Request Scheduling</td>
			<td></td>
			<td>DURATION</td>
		</tr>
		<tr>
			<td>Sending request</td>
			<td>
				<div class="graph-row">
					<span class="graph-bar rinit" style="left: ${time.rinit.left}%; right: ${time.rinit.right}%;">​</span>
				</div>
			</td>
			<td>
				<div class="graph-bar-title">${time.rinit.s}&nbsp;s</div>
			</td>
		</tr>
		<tr class="graph-table-header">
			<td role="heading">Request processing</td>
			<td></td>
			<td>DURATION</td>
		</tr>
		<tr>
			<td></td>
			<td>
				<div class="graph-row">
					<span class="graph-bar" style="left: ${time.req.left}%; right: ${time.req.right}%;">​</span>
				</div>
			</td>
			<td>
				<div class="graph-bar-title">${time.req.s}&nbsp;s</div>
			</td>
		</tr>
		<tr class="graph-table-header">
			<td role="heading">Execution</td>
			<td></td>
			<td>DURATION</td>
		</tr>
		<tr>
			<td>PHP</td>
			<td>
				<div class="graph-row">
					<span class="graph-bar" style="left: ${time.functions.left}%; right: ${time.functions.right}%;">​</span>
				</div>
			</td>
			<td>
				<div class="graph-bar-title">${time.functions.s}&nbsp;s</div>
			</td>
		</tr>
		<tr class="graph-footer">
			<td>
				Total
			</td>
			<td></td>
			<td>${time.total.s}&nbsp;s</td>
		</tr>
		<tr class="graph-table-header">
			<td colspan="3">
				<hr class="break" />
			</td>
		</tr>
		<tr class="graph-table-header">
			<td role="heading">Memory</td>
			<td></td>
			<td>AMOUNT</td>
		</tr>
		<tr>
			<td>Usage</td>
			<td>
				<div class="graph-row">
					<span class="graph-bar" style="left: 0%; right: ${100 - mem.usagePct}%;">​</span>
				</div>
			</td>
			<td>
				<div class="graph-bar-title">${mem.usage}&nbsp;MB</div>
			</td>
		</tr>
		<tr>
			<td>Peak usage</td>
			<td>
				<div class="graph-row">
					<span class="graph-bar" style="left: 0%; right: ${100 - mem.peakPct}%;">​</span>
				</div>
			</td>
			<td>
				<div class="graph-bar-title">${mem.peak}&nbsp;MB</div>
			</td>
		</tr>
		<tr class="graph-footer">
			<td>Limit</td>
			<td>
				<div class="graph-row"></div>
			</td>
			<td>
				<div class="graph-bar-title">${mem.limit}&nbsp;MB</div>
			</td>
		</tr>
		<tr class="graph-table-header">
			<td colspan="3">
				<hr class="break" />
			</td>
		</tr>
		<tr class="graph-table-header">
			<td role="heading">Process</td>
			<td></td>
			<td>DETAILS</td>
		</tr>
		<tr>
			<td>Host</td>
			<td colspan="2">
				<div class="graph-bar-title">${process.node_name}</div>
			</td>
		</tr>
		<tr>
			<td>PID</td>
			<td colspan="2">
				<div class="graph-bar-title">${process.pid}</div>
			</td>
		</tr>
	</table>`
}

export default class InfoPage {
	constructor(request, prefix) {
		// console.log('infopage::construct', request);
		this.Request = request;
		this.method = '';
		this.target = '';
		this.httpResponseCode = '';
		this.memory = { limit: 0, usage: 0, peak: 0};
		this.time = {total: 0, functions: 0, rinit: 0};

		this.content = dom.div({
			id: prefix + '-pg-info',
			text: "Info page"
		});
	}

	get() {
		// console.log('infopage::get');
		return this.content;
	}

	init(data) {
		// console.log('infopage::rinit', data);
		/*
			"method": "GET",
			"node_name": "zray",
			"pid": 350,
			"request_id": 1,
			"target": "http://zray.local:80/test.php?zraytok=cPGtYAAAAAAwo11VjQCbfgAAAL8",
			"time_sec": 1621925884,
			"time_usec": 966956,
			"type": "RINIT"
		*/
		this.method = data.method;
		this.target = data.target;
		let startMs = The.getRequestTiming(data.target);
		this.rinitTime = (startMs ? Date.now() - startMs : 0);
		this.node_name = data.node_name || "unknown";
		this.pid = data.pid || 0;

		this.build();
	}

	shutdown(data) {
		// console.log('infopage::shutdown', data);
		/*
			http_response_code: 200
			memory_limit: 134217728
			memory_peak_usage: 3646136
			memory_usage: 3207696
			request_id: 40
			total_time: 123.67415428161621
			type: "RSHUTDOWN"
		*/
		this.httpResponseCode = parseInt(data.http_response_code);
		this.memory = {
			limit: parseInt(data.memory_limit),
			usage: parseInt(data.memory_usage),
			peak: parseInt(data.memory_peak_usage),
		};

		this.time.total = parseFloat(data.total_time); //.toFixed(3);

		this.build();
	}

	functionsTime(time) {
		// console.log('infopage::functionsTime', time);
		this.time.functions = time;
	}

	build() {
		// console.log('infopage::build');
		let total = this.time.rinit + this.time.total;
		let start = 0;
		const percentage = (time) => {
			let left = start;
			start += (time / total * 100);
			return {
				s: (time / 1000).toFixed(3),
				left: left,
				right: 100 - start
			}
		};

		this.content.innerHTML = '';
		dom.create(
			template.page(
				{
					method: this.method,
					url: this.target,
					code: this.httpResponseCode
				},
				{
					rinit: percentage(this.time.rinit),
					req: percentage(this.time.total - this.time.functions),
					functions: percentage(this.time.functions),
					total: { s: (total / 1000).toFixed(3) }
				},
				{
					limit: (this.memory.limit / 1048576).toFixed(2),
					usage: (this.memory.usage / 1048576).toFixed(2),
					peak: (this.memory.peak / 1048576).toFixed(2),
					usagePct: (this.memory.usage * 100 / this.memory.limit).toFixed(3),
					peakPct: (this.memory.peak * 100 / this.memory.limit).toFixed(3),
				},
				{
					node_name: this.node_name,
					pid: this.pid
				}
			),
			{ appendTo: this.content }
		);

	}
}