import dom from "../elements/Dom.js";
import Collapsible from "../elements/Collapsible.js";
import Sortable from "../elements/Sortable.js";
import Tabable from "../elements/Tabable.js";

export default class PluginsPage {
	constructor(request, namespace, prefix) {
		// console.log('PluginsPage[' + namespace + ']::construct', request);
		this.request = request;
		this.namespace = namespace;
		this.name = prefix + '-pg-' + namespace.toLowerCase();
		this.tabs = new Tabable({}, namespace, prefix + '-pg');
		this.data = {};
	}

	get() {
		// console.log('PluginsPage['+ this.namespace +']::get');
		return this.tabs.get();
	}

	add(data) {
		// data:
		// 	dispatcher: "N/A"
		// 	name: "kernel.request"
		// 	propagation stopped: false
		// 	type: "Symfony\Component\HttpKernel\Event\RequestEvent"
		// key: "events"
		// name_space: "symfony"
		if ("data" in data) {
			let key = data.key || "unknown";
			key = key[0].toUpperCase() + key.slice(1); // uppercase first char

			if (key in this.data === false) {
				this.data[key] = [];
			}
			this.data[key].push(data.data);
		}
	}

	build() {
		for (let key in this.data) {
			let tab;
			if (this.data[key].length == 1) {
				// tree
				let tree = new Collapsible({}, this.name + '-' + key.toLowerCase());
				tree.add(this.data[key].pop());
				tab = tree.get();
			}
			else {
				// table
				let headers = {};
				let keys = Object.keys(this.data[key][0]);
				for (let k in keys) {
					headers[keys[k]] = {};
				}
				// console.log(headers);
				let table = new Sortable(headers, this.name + '-' + key.toLowerCase());
				table.addRows(this.data[key]);
				tab = table.get();
				// tab = dom.div({
				// 	text: JSON.stringify(this.data[key])
				// });
			}
			this.tabs.append(key, tab);
			this.tabs.enable(key);
		}

		this.tabs.initClick(Object.keys(this.data)[0]);
	}
}